#ifndef A8_ASYNC_TCPCLIENT_H
#define A8_ASYNC_TCPCLIENT_H

#include <atomic>
#include <a8/timer_attacher.h>
#include <a8/epolleventhandler.h>

namespace a8
{
    class IoLoop;
    class AsyncTcpClient : public EpollEventHandler
    {
    public:
        std::function<void (a8::AsyncTcpClient*, int)> on_error;
        std::function<void (a8::AsyncTcpClient*)> on_connect;
        std::function<void (a8::AsyncTcpClient*)> on_disconnect;
        std::function<void (a8::AsyncTcpClient*, char*, unsigned int)> on_socketread;
        std::string remote_address;
        int remote_port = 0;
        int timeout_ms = 1000 * 10;
        a8::XTimerAttacher connect_timer_attacher;
        std::atomic<long long> ref_count;

        AsyncTcpClient();
        virtual ~AsyncTcpClient();

        void Open();
        void Close();
        bool IsActive();
        bool Connected();
        void SendBuff(const char* buff, unsigned int bufflen);

        void DoConnect();

        virtual void SetEpollFd(int epoll_fd) override;
        virtual void DoRecv() override;
        virtual void DoSend() override;
        virtual void DoError() override;

    private:

        void DoAsyncConnect();
        void DoAsyncClose();
        void ClearSendBuff();

    private:
        volatile int epoll_fd_ = a8::INVALID_FD;
        volatile int socket_ = a8::INVALID_SOCKET;
        volatile bool sending_ = false;
        volatile bool connected_ = false;
        std::mutex* send_buffer_mutex_ = nullptr;
        a8::SendQueueNode *top_node_ = nullptr;
        a8::SendQueueNode *bot_node_ = nullptr;
        a8::SendQueueNode *work_node_ = nullptr;

        void SetActive(bool active);
        void NotifyEpollSend();
        void AsyncSend();

        friend class IoLoop;
    };
}

#endif
