#ifndef A8_IOLOOP_H
#define A8_IOLOOP_H

#include <atomic>
#include <a8/a8.h>
#include <a8/types.h>

namespace a8
{
    class AsyncTcpClient;
    struct IoLoopThreadContext;
    class IoLoop : public a8::Singleton<IoLoop>
    {
    private:
        IoLoop() {};
        friend class a8::Singleton<IoLoop>;

    public:
        std::atomic<long long> run_times = {0};
        std::atomic<long long> timer_times = {0};
        std::atomic<long long> event_times = {0};
        std::atomic<long long> free_times = {0};
        std::atomic<long long> shutdown_times = {0};
        std::atomic<long long> connect_times = {0};
        std::atomic<long long> close_times = {0};
        std::atomic<long long> send_times = {0};
        std::atomic<long long> recv_times = {0};
        std::atomic<long long> error_times = {0};
        std::atomic<long long> immsg_times = {0};

    public:
        void Init(int thread_num);
        void UnInit();

        AsyncTcpClient* CreateAsyncTcpClient();
        void DestoryAsyncTcpClient(AsyncTcpClient* client);

    private:
        void WorkerThreadProc(IoLoopThreadContext* context);
        void AddIMMsg(IoLoopThreadContext* context, int imcmd, a8::XParams params);
        void ProcessIMMsg(IoLoopThreadContext* context);
        void OnEvent(void* context, unsigned long long data);
        void OnTimer(void* context, unsigned long long data);
        void PostAsyncConnect(AsyncTcpClient* client, int timeout_ms);
        void PostAsyncClose(AsyncTcpClient* client);

        void _IMFreeClient(IoLoopThreadContext* context, a8::XParams& param);
        void _IMShutdown(IoLoopThreadContext* context, a8::XParams& param);
        void _IMAsyncConnect(IoLoopThreadContext* context, a8::XParams& param);
        void _IMAsyncClose(IoLoopThreadContext* context, a8::XParams& param);

    private:
        std::atomic<unsigned long long> event_id_;
        int thread_num_ = 1;
        int max_client_num_ = 10000;
        volatile bool worker_thread_shutdown_ = false;
        std::vector<IoLoopThreadContext*> thread_contexts_;
        std::vector<std::thread*> worker_threads_;

        friend class AsyncTcpClient;
    };

}

#endif
