#include <unistd.h>

#include <sys/epoll.h>
#include <sys/timerfd.h>

#include <a8/a8.h>
#include <a8/timerfd.h>

namespace a8
{

    TimerFD::TimerFD()
    {
        fd_ = ::timerfd_create(CLOCK_MONOTONIC, TFD_NONBLOCK | TFD_CLOEXEC);
        if (fd_ == a8::INVALID_FD) {
            abort();
        }
    }

    TimerFD::~TimerFD()
    {
        UnInit();
    }

    void TimerFD::Init(void* context)
    {
        context_ = context;
    }

    void TimerFD::UnInit()
    {
        if (fd_ != a8::INVALID_FD) {
            struct epoll_event ev;
            ::epoll_ctl(epoll_fd_, EPOLL_CTL_DEL, fd_, &ev);
            ::close(fd_);
            fd_ = a8::INVALID_FD;
        }
    }

    void TimerFD::Start(long long ms)
    {
        itimerspec new_value;

        new_value.it_interval.tv_sec = ms / 1000;
	new_value.it_interval.tv_nsec = (ms % 1000) * 1000 * 1000;

	new_value.it_value.tv_sec = ms / 1000;
	new_value.it_value.tv_nsec = (ms % 1000) * 1000 * 1000;

        if (timerfd_settime(fd_, 0, &new_value, nullptr) != 0) {
            abort();
        }
    }

    void TimerFD::Stop()
    {
        itimerspec new_value;

        new_value.it_interval.tv_sec = 0;
	new_value.it_interval.tv_nsec = 0;

	new_value.it_value.tv_sec = 0;
	new_value.it_value.tv_nsec = 0;

        if (timerfd_settime(fd_, 0, &new_value, nullptr) != 0) {
            abort();
        }
    }

    void TimerFD::SetEpollFd(int epoll_fd)
    {
        struct epoll_event ev;
        ev.data.fd = fd_;
        ev.events = EPOLLIN | EPOLLET;
        ev.data.ptr = this;
        int ret = ::epoll_ctl(epoll_fd, EPOLL_CTL_ADD, fd_, &ev);
        if (ret != 0) {
            abort();
        }
        epoll_fd_ = epoll_fd;
    }

    void TimerFD::DoRecv()
    {
        unsigned long long value = 0;
        ::read(fd_, &value, sizeof(value));
        if (OnTimer) {
            OnTimer(context_, value);
        }
    }

    void TimerFD::DoSend()
    {
        abort();
    }

    void TimerFD::DoError()
    {
        abort();
    }

}
