#include <assert.h>
#include <sys/time.h>
#include <unistd.h>

#include <a8/a8.h>
#include <a8/uuid.h>

namespace a8
{
    namespace uuid
    {

        long long SnowFlake::FetchTime(long long value)
        {
            return (value >> 22)  + a8::uuid::A8_EPOCH;
        }

        int SnowFlake::FetchMachineId(long long value)
        {
            long long machineid = (value >> SEQUNCE_ID_BIT_NUM) & (MAX_MACHINE_ID);
            return machineid;
        }

        int SnowFlake::FetchSequnceId(long long value)
        {
            return value & MAX_SEQUNCE_ID;
        }

        long long SnowFlake::Generate()
        {
            long long value = 0;
            long long time = a8::GetMilliSecond();
            if (time == last_generate_tick) {//同一微妙内
                sequence_id++;
                if (sequence_id >= MAX_SEQUNCE_ID) { //计数已经用完
                    while (time <= last_generate_tick) {
                        ::usleep(1);
                        time = a8::GetMilliSecond();
                    }
                    sequence_id = 0;
                }
            } else {
                sequence_id = 0;
            }
            last_generate_tick = time;
            assert(time > a8::uuid::A8_EPOCH);
            time = time - a8::uuid::A8_EPOCH;

            // 保留后41位时间
            value = time << 22;

            // 中间12位是机器ID
            value |= (machine_id & MAX_MACHINE_ID) << SEQUNCE_ID_BIT_NUM;

            // 最后10位是sequenceID
            value |= sequence_id & MAX_SEQUNCE_ID;

            return value;
        }

        void SnowFlake::SetMachineId(unsigned short machineid)
        {
            if (machineid > MAX_MACHINE_ID || machineid < 1) {
                abort();
            }
            machine_id = machineid;
        }
    }
}
